use egui::ImageSource;
use strum::EnumIter;

#[derive(Hash, Debug, Clone, PartialEq, Eq, Copy, Default, EnumIter)]
pub enum Kind {
    Playlist,
    #[default]
    Album,
    Song,
    Artist,
    Genre,
}

#[derive(Clone, Debug)]
pub struct Entry {
    pub kind: Kind,
    pub title: String,
    pub subtitle: String,
    /// image url because I can't be bothered to use lifetimes everywhere
    pub img_url: Option<String>,
}

impl Entry {
    pub fn new(kind: Kind, title: &str, subtitle: &str, img: Option<String>) -> Self {
        Self {
            kind,
            title: title.to_owned(),
            subtitle: subtitle.to_owned(),
            img_url: img,
        }
    }
    pub fn img(&self) -> ImageSource {
        if let Some(url) = &self.img_url {
            ImageSource::Uri(std::borrow::Cow::from(url))
        } else {
            ImageSource::Bytes {
                uri: std::borrow::Cow::Borrowed("builtin://icon_256.jpg"),
                bytes: egui::load::Bytes::Static(super::ICON_RAW),
            }
        }
    }
}

impl std::fmt::Display for Entry {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", self.title)
    }
}

impl std::fmt::Display for Kind {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "{}",
            match self {
                Self::Song => "Song",
                Self::Genre => "Genre",
                Self::Album => "Album",
                Self::Playlist => "Playlist",
                Self::Artist => "Artist",
            }
        )
    }
}
